#sidebar {
    dock: left;
    width: auto;
}

.container {
    height: auto;
    width: auto;
}

Checkbox {
    height: auto;
    width: auto;
}

.label {
    height: 3;
    content-align: center middle;
    width: auto;
}
